local i18n = QBL.i18n[QBL.locale].Main

local main_f = QBL.frames.main
local version = QBL.version
local cfg_defaults = QBL.cfg_defaults

local init_menu = QBL.lib.Interface.init_menu

function QBL.events:ADDON_LOADED(addon, ...)
	if addon == "Questionable" then
		if QBL_cfg then
			-- Remove outdated config options.
			for k in pairs(QBL_cfg) do
				if cfg_defaults[k] == nil then QBL_cfg[k] = nil end
			end
			-- Add missing config options.
			for k, v in pairs(cfg_defaults) do
				if QBL_cfg[k] == nil then QBL_cfg[k] = v end
			end
		-- Else, create the config table.
		else QBL_cfg = QBL:copy_cfg() end

		-- Set up the Q? configuration menu.
		init_menu()

		-- Register the needed events and timers for each option.
		for option in pairs(QBL_cfg) do
			-- Enabled options are either true or > 1.
			if (QBL_cfg[option] == true
					or (type(QBL_cfg[option]) == "number"
						and QBL_cfg[option] > 1))
				and QBL.srv.opt[option]
			then
				QBL:enable_opt(option)
			end
		end
		for event, _ in pairs(QBL.startup) do main_f:RegisterEvent(event) end

		if QBL_cfg.intro then
			QBL:print("|cffBE81F7-- Questionable v" .. version .. " --|r")
			QBL:print(i18n.INTRO_HELP)
			local meta_str = i18n.INTRO_SKIP
			if QBL_cfg.reverse_meta then meta_str = i18n.INTRO_ENABLE end
			QBL:print(meta_str)
		end

		main_f:UnregisterEvent("ADDON_LOADED")
	end
end

main_f:RegisterEvent("ADDON_LOADED")

-- Passes the event to the correct handler in the events table.
main_f:SetScript("OnEvent", function(self, event, ...)
	QBL.events[event](self, ...) end)
