local i18n = QBL.i18n[QBL.locale].lib.Groups

local _prefix = QBL.prefix
local try_sound = QBL.util.Config.try_sound
local fmt_notice = QBL.util.Text.fmt_notice
local wowhead_url = QBL.util.Links.wowhead_url

local quest = {}
local player = UnitName("player") .. "-" .. gsub(GetRealmName(), "%s", "")

-- Returns the player name from 'Player-Realm' strings.
local function get_player(str)
	local dash = string.find(str, "-")
	if dash then return str:sub(1, dash - 1)
	else return str end
end

-- Returns true if 'person' is a friend.
local function is_friend(person)
	local friends = {}
	local person = get_player(person)

	if QBL_cfg.bnet_friends then
		for friend = 1, select(2, BNGetNumFriends()) do
			for toon = 1, BNGetNumFriendGameAccounts(friend) do
				local _, name, game = BNGetFriendGameAccountInfo(friend, toon)
				if game == "WoW" then table.insert(friends, name) end
			end
		end
	end
	for friend = 1, select(2, GetNumFriends()) do
		local name = get_player(GetFriendInfo(friend))
		table.insert(friends, name)
	end

	local value
	for _, friend in pairs(friends) do
		if person == friend then
			value = true
			break
		end
	end

	if value then return true
	else return false end
end

-- Routes party/raid chat messages, based on configuration options.
function QBL.events:CHAT_MSG_ADDON(prefix, chat, channel, sender, ...)
	if sender ~= player and prefix == _prefix then
		local type = string.sub(chat, 1, 3)
		local msg = string.sub(chat, 4)
		local name = get_player(sender)
		local my_friend = is_friend(sender)

		if QBL_cfg.tracker then
			if type == "ADD" then
				return QBL:print(fmt_notice(i18n.Q_ADD), name, "->", msg)
			elseif type == "PRG" then
				return QBL:print(fmt_notice(i18n.OBJ_PROG), name, "->", msg)
			elseif type == "OBJ" then
				return QBL:print(fmt_notice(i18n.OBJ_COMP, "GOOD"), name, "->", msg)
			elseif type == "CPL" then
				try_sound("pickupring")
				return QBL:print(fmt_notice(i18n.Q_COMP, "GOOD"), name, "->", msg)
			elseif type == "FAL" then
				try_sound("DeathThudSmallSnow")
				return QBL:print(fmt_notice(i18n.Q_FAIL, "BAD"), name, "->", msg)
			elseif type == "TRN" then
				return QBL:print(fmt_notice(i18n.Q_TURN), name, "->", msg)
			end
		end

		if QBL_cfg.promote_friend then
			if type == "PRO" then
				return QBL:print(fmt_notice(i18n.F_PROMOTE, "WARN"), name, "->", msg)
			-- Sent from lib/Interface. Gives Group Leader to sender.
			elseif type == "GGL"
				and my_friend
				and UnitIsGroupLeader("player")
			then
				PromoteToLeader(name)
				QBL:send_msg("PRO" .. name)
				return QBL:print(fmt_notice(i18n.F_PROMOTE, "WARN"), "You ->", name)
			end
		end

		if QBL_cfg.follow_friend then
			-- Sent from lib/Interface. Follows sender.
			if type == "FOL"
				and msg == UnitName("player")
				and my_friend
			then
				FollowUnit(name)
				QBL:send_msg("ACK" .. name)
				return QBL:print(fmt_notice(i18n.F_FOLLOW, "WARN"), "You ->", name)
			elseif type == "ACK" and msg == UnitName("player") then
				return QBL:print(fmt_notice(i18n.F_FOLLOW, "WARN"), name, "-> You")
			end
		end

		if QBL_cfg.invite_friend then
			if type == "INV" then
				return QBL:print(fmt_notice(i18n.F_INVITE, "GOOD"), name, "->", msg)
			end
		end

		-- Sent from util/Links. Gives Needed quest to sender.
		if type == "NED" then
			local idx
			for i = 1, GetNumQuestLogEntries() do
				local id = select(8, GetQuestLogTitle(i))
				if tonumber(msg) == tonumber(id) then idx = i end
			end
			if idx then
				SelectQuestLogEntry(idx)
				QuestLogPushQuest()
				return QBL:print(fmt_notice(i18n.Q_SHARED), name, "->", wowhead_url(idx))
			end
		end
	end
end

function QBL.events:CHAT_MSG_WHISPER(msg, sender, ...)
	local my_friend = is_friend(sender)
	local name = get_player(sender)
	msg = string.lower(msg)
	if QBL_cfg.promote_friend
		and (msg == "qbl promote" or msg == "qbl p")
		and UnitIsGroupLeader("player")
		and my_friend
	then
		PromoteToLeader(name)
		QBL:send_msg("PRO" .. name)
		QBL:print(fmt_notice(i18n.F_PROMOTE, "WARN"), "You ->", name)
	elseif QBL_cfg.invite_friend
		and (msg == "qbl invite" or msg == "qbl i")
		and UnitIsGroupLeader("player")
		and my_friend
	then
		InviteUnit(sender)
		QBL:send_msg("INV" .. name)
		QBL:print(fmt_notice(i18n.F_INVITE, "GOOD"), "You -> ", name)
	elseif QBL_cfg.follow_friend
		and (msg == "qbl follow" or msg == "qbl f")
		and my_friend
	then
		QBL:print(fmt_notice(i18n.F_FOLLOW, "WARN"), "You -> ", name)
		FollowUnit(name)
		if IsInGroup() then QBL:send_msg("ACK" .. name)
		else SendChatMessage("Trying to follow!", "WHISPER", nil, name) end
	end
end

function QBL.events:CHAT_MSG_PARTY(msg, sender, ...)
	local my_friend = is_friend(sender)
	local name = get_player(sender)
	msg = string.lower(msg)
	if (msg == "qbl promote" or msg == "qbl p")
		and QBL_cfg.promote_friend
		and UnitIsGroupLeader("player")
		and my_friend
	then
		PromoteToLeader(name)
		QBL:send_msg("PRO" .. name)
		QBL:print(fmt_notice(i18n.F_PROMOTE, "WARN"), "You ->", name)
	elseif (msg == "qbl follow" or msg == "qbl f")
		and QBL_cfg.follow_friend
		and my_friend
	then
		QBL:print(fmt_notice(i18n.F_FOLLOW, "WARN"), "You -> ", name)
		FollowUnit(name)
		QBL:send_msg("ACK" .. name)
	end
end

function QBL.events:CHAT_MSG_PARTY_LEADER(msg, sender, ...)
	local name = get_player(sender)
	msg = string.lower(msg)
	if (msg == "qbl follow" or msg == "qbl f")
		and QBL_cfg.follow_friend
		and is_friend(sender)
	then
		QBL:print(fmt_notice(i18n.F_FOLLOW, "WARN"), "You -> ", name)
		FollowUnit(name)
		QBL:send_msg("ACK" .. name)
	end
end

function QBL.events:QUEST_LOG_UPDATE()
	for i = 1, GetNumQuestLogEntries() do
		local txt, level, _, header, _, complete = GetQuestLogTitle(i)

		if level > 0 and header ~= 1 then
			if quest[txt] then
				if complete and quest[txt].complete ~= complete then
					quest[txt].complete = complete
					if complete == -1 then
						QBL:send_msg("FAL" .. quest[txt].link)
						try_sound("DeathThudSmallWood")
					elseif UnitExists("party1") then
						QBL:send_msg("CPL" .. quest[txt].link)
						try_sound("putdownring")
					else try_sound("putdownring") end
				else quest[txt].complete = complete end
			else
				SelectQuestLogEntry(i)
				quest[txt] = {complete = complete, link = wowhead_url(i)}

				if quest[txt].link
				then QBL:send_msg("ADD" .. quest[txt].link)
				else QBL:send_msg("ADD" .. txt) end
			end
		end
	end
end

function QBL.events:UI_INFO_MESSAGE(msg, ...)
	obj = ...
	-- Extracts the quest objective text from the info message.
	local txt = gsub(obj, "(.*):%s*([-%d]+)%s*/%s*([-%d]+)%s*$", "%1", 1)
	if txt ~= obj then
		local _, _, _, have, needed = string.find(
			obj, "(.*):%s*([-%d]+)%s*/%s*([-%d]+)%s*$")

		local missing
		if needed and have then missing = needed - have end
		if missing and missing > 0 then
			QBL:send_msg("PRG" .. obj)
			try_sound("GO_Spires_Quest_PickUp_Illusion_Charm")
		else
			QBL:send_msg("OBJ" .. obj)
 			try_sound("PickupBook")
		end
	end
end

function QBL.events:QUEST_TURNED_IN()
	local txt = GetTitleText()
	if quest[txt] and quest[txt].link
	then QBL:send_msg("TRN" .. quest[txt].link)
	else QBL:send_msg("TRN" .. txt) end
end

function QBL.events:GROUP_ROSTER_UPDATE(...)
	if IsInGroup() then
		StaticPopup_Hide("PARTY_INVITE")
		StaticPopup_Hide("PARTY_INVITE_XREALM")
	end
end

function QBL.events:PARTY_INVITE_REQUEST(sender, ...)
	if QBL_cfg.accept_invite
		and is_friend(sender)
	then AcceptGroup() end
end

function QBL.events:PLAYER_ENTERING_WORLD()
	self:RegisterEvent("QUEST_LOG_UPDATE")
end
function QBL.events:PLAYER_LEAVING_WORLD()
	self:UnregisterEvent("QUEST_LOG_UPDATE")
end

QBL.srv.tracker = {"CHAT_MSG_ADDON", "PLAYER_ENTERING_WORLD",
	"PLAYER_LEAVING_WORLD", "QUEST_LOG_UPDATE", "QUEST_TURNED_IN",
	"UI_INFO_MESSAGE"}
QBL.srv.follow_friend = {"CHAT_MSG_ADDON", "CHAT_MSG_WHISPER",
	"CHAT_MSG_PARTY", "CHAT_MSG_PARTY_LEADER"}
QBL.srv.promote_friend = { "CHAT_MSG_ADDON", "CHAT_MSG_WHISPER",
	"CHAT_MSG_PARTY"}
QBL.srv.accept_invite = {"GROUP_ROSTER_UPDATE", "PARTY_INVITE_REQUEST"}
QBL.srv.invite_friend = {"CHAT_MSG_ADDON", "CHAT_MSG_WHISPER"}
